# -*- coding: UTF-8 -*-

class CashConvertersConfig(object):
    """ Класс для получения настроек конвертеров для кассы """
    def __init__(self, config, section):
        self.config = config
        self.section = section

    def getInt(self, name, defaultValue):
        return self.config.getInt(self.section, name, defaultValue)

    def getBool(self, name, defaultValue):
        return self.config.getBool(self.section, name, defaultValue)

    def getBoolWithoutDefaultValue(self, name):
        defaultValue = True
        return self.getBool(name, defaultValue)

    def getString(self, name, defaultValue):
        return self.config.getString(self.section, name, defaultValue)

    def getParamValue(self, key, defaultValue=None):
        return self.getString(key, defaultValue)

    def getFormat(self, defaultValue):
        return self.getString('fileType', defaultValue)

    def getCharset(self, converterName):
        defaultValue = 'cp1251'
        if converterName in ['smag', 'SUPERMAG', 'pilot-original', 'PILOT-ORIGINAL']:
            defaultValue = 'cp866'
        return self.getString('charset', defaultValue)

    def readSubsection(self, subsectionName):
        fullName = self.section + '.' + subsectionName
        section = self.config.getOptions(fullName)
        sectionMap = {}
        
        for m in section:
            sectionMap[m] = self.config.getString(fullName, m, '')
        
        return sectionMap

    def getGroup2CatalogMapper(self):
        return self.readSubsection('group2catalog')

    def getValutMapper(self):
        return self.readSubsection('valut')
    
    def getMeasureMapper(self):
        return self.readSubsection('measure')
    
    def getRolesMapper(self):
        return self.readSubsection('roles')
    
    def getPaymentMapper(self):
        return self.readSubsection('payment')

    def getOracleMeasureMappers(self):
        mappers = dict()
        for s in ['upload', 'unload']:
            sectionMap = dict()
            for param in self.config.getOptions(s + '.converter.oracle.measure'):
                sectionMap[param] = self.config.getString(s + '.converter.oracle.measure', param)
            mappers[s] = sectionMap
        return mappers

    def getSmagXmlMeasureMapper(self):
        return self.readSubsection('measureMapper')

    def getSmagXmlTmcTypeMapper(self):
        return self.readSubsection('tmcTypeMapper')

    def getSmagXmlPaymentMapper(self):
        return self.readSubsection('paymentMapper')

    def getSmagXmlQtyOptionsMapper(self):
        return self.readSubsection('quantityOptionsMapper')

    def getSmagXmlTmcRestrictionsMapper(self):
        return self.readSubsection('tmcRestrictionsMapper')
