# -*- coding: UTF-8 -*-

import os
import sys
sys.path.append(os.path.dirname(__file__))
import dbConnection
from shiftUnloader import ShiftUnloader
from documentUnloader import DocumentUnloader
from securitylogUnloader import SecurityLogUnloader


def normalizeCortege(cortege):
    tempList = []
    for dictParams in cortege:
        for param in dictParams.values():
            tempList.append(str(param))
    return tuple(tempList)


class Unloader(object):
    def __init__(self, node, log):
        self.connection = dbConnection.connect(node.reader.host, node.reader.port, node.reader.username, node.reader.password, node.reader.database, node.reader.dbtype)
        self.shiftUnloader = ShiftUnloader(self.connection, log, node.id)
        self.documentUnloader = DocumentUnloader(self.connection, log)
        self.securitylogUnloader = SecurityLogUnloader(self.connection, log)

    def __del__(self):
        if hasattr(self, 'connection') and self.connection is not None:
            self.connection.close()

    def setFlagUnload(self, shifts, documents):
        cursor = self.connection.cursor()
        if documents:
            cursor.execute("UPDATE `document` SET `changed` = 0 WHERE `documentid` IN %s", (documents,))
        if shifts:
            cursor.execute("UPDATE `workshift` SET `changed` = 0 WHERE `workshiftid` IN %s", (shifts,))
        self.connection.commit()

    def unloadChanged(self, cashCode, closedShifts):
        shiftIds = normalizeCortege(self.shiftUnloader.getChangedShifts(cashCode, closedShifts))
        unloadShifts = lambda : self.shiftUnloader.unloadShifts(shiftIds)
        documentIds = normalizeCortege(self.documentUnloader.getChangedDocuments(shiftIds))
        unloadDocuments = lambda : self.documentUnloader.unloadChangedDocuments(documentIds)
        unloadSecurityLogs = lambda : self.securitylogUnloader.unloadSecurityLogs(shiftIds)
        flagUnload = lambda : self.setFlagUnload(shiftIds, documentIds)
        return unloadShifts, unloadDocuments, unloadSecurityLogs, flagUnload

    def unloadChangedShifts(self, cashCode, closedShifts):
        shiftIds = normalizeCortege(self.shiftUnloader.getChangedShifts(cashCode, closedShifts))
        unloadShifts = lambda : self.shiftUnloader.unloadShifts(shiftIds)
        unloadDocuments = lambda : self.documentUnloader.unloadDocuments(shiftIds)
        unloadSecurityLogs = lambda : self.securitylogUnloader.unloadSecurityLogs(shiftIds)
        documentIds = ()
        flagUnload = lambda : self.setFlagUnload(shiftIds, documentIds)
        return unloadShifts, unloadDocuments, unloadSecurityLogs, flagUnload

    def unloadByShifts(self, cashCode, fromShift, toShift, closedShifts):
        shiftIds = normalizeCortege(self.shiftUnloader.getShiftsByShiftnum(cashCode, fromShift, toShift, closedShifts))
        unloadShifts = lambda : self.shiftUnloader.unloadShifts(shiftIds)
        unloadDocuments = lambda : self.documentUnloader.unloadDocuments(shiftIds)
        unloadSecurityLogs = lambda : self.securitylogUnloader.unloadSecurityLogs(shiftIds)
        return unloadShifts, unloadDocuments, unloadSecurityLogs, None

    def unloadByShiftsAndSetChanged(self, cashCode, fromShift, toShift, closedShifts):
        shiftIds = normalizeCortege(self.shiftUnloader.getShiftsByShiftnum(cashCode, fromShift, toShift, closedShifts))
        unloadShifts = lambda : self.shiftUnloader.unloadShifts(shiftIds)
        documentIds = normalizeCortege(self.documentUnloader.getChangedDocuments(shiftIds))
        unloadDocuments = lambda : self.documentUnloader.unloadDocuments(shiftIds)
        unloadSecurityLogs = lambda : self.securitylogUnloader.unloadSecurityLogs(shiftIds)
        flagUnload = lambda : self.setFlagUnload(shiftIds, documentIds)
        return unloadShifts, unloadDocuments, unloadSecurityLogs, flagUnload

    def unloadByShiftDates(self, cashCode, fromDate, toDate, closedShifts):
        shiftIds = normalizeCortege(self.shiftUnloader.getShiftsByDate(cashCode, fromDate, toDate, closedShifts))
        unloadShifts = lambda : self.shiftUnloader.unloadShifts(shiftIds)
        unloadDocuments = lambda : self.documentUnloader.unloadDocuments(shiftIds)
        unloadSecurityLogs = lambda : self.securitylogUnloader.unloadSecurityLogs(shiftIds)
        return unloadShifts, unloadDocuments, unloadSecurityLogs, None

    def unloadByDocumentDates(self, cashCode, fromDate, toDate):
        shiftIds = normalizeCortege(self.shiftUnloader.getShiftsByDocumentDate(cashCode, fromDate, toDate))
        unloadShifts = lambda : self.shiftUnloader.unloadShifts(shiftIds)
        unloadDocuments = lambda : self.documentUnloader.unloadDocumentsByDate(shiftIds, fromDate, toDate)
        unloadSecurityLogs = lambda : self.securitylogUnloader.unloadSecurityLogsByDate(shiftIds, fromDate, toDate)
        return unloadShifts, unloadDocuments, unloadSecurityLogs, None
    
    def unloadByShop(self, fromDate, toDate, cashList):
        shiftIds = normalizeCortege(self.shiftUnloader.getShiftsByDocumentDateForShop(fromDate, toDate, cashList))
        unloadShifts = lambda : self.shiftUnloader.unloadShifts(shiftIds)
        unloadDocuments = lambda : self.documentUnloader.unloadDocumentsByDate(shiftIds, fromDate, toDate)
        unloadSecurityLogs = lambda : self.securitylogUnloader.unloadSecurityLogsByDate(shiftIds, fromDate, toDate)
        return unloadShifts, unloadDocuments, unloadSecurityLogs, None

    def unloadByShiftsForShop(self, fromShift, toShift, cashList, closedShifts):
        shiftIds = normalizeCortege(self.shiftUnloader.getShiftsByShiftsForShop(fromShift, toShift, cashList, closedShifts))
        unloadShifts = lambda : self.shiftUnloader.unloadShifts(shiftIds)
        unloadDocuments = lambda : self.documentUnloader.unloadDocuments(shiftIds)
        unloadSecurityLogs = lambda : self.securitylogUnloader.unloadSecurityLogs(shiftIds)
        return unloadShifts, unloadDocuments, unloadSecurityLogs, None

    def unloadOpenedShiftsToDay(self, cashCode):
        shiftIds = normalizeCortege(self.shiftUnloader.getOpenedShiftsToDay(cashCode))
        unloadShifts = lambda : self.shiftUnloader.unloadShifts(shiftIds)
        unloadDocuments = lambda : self.documentUnloader.unloadDocuments(shiftIds)
        unloadSecurityLogs = lambda : self.securitylogUnloader.unloadSecurityLogs(shiftIds)
        return unloadShifts, unloadDocuments, unloadSecurityLogs, None
