# -*- coding: UTF-8 -*-

import json

from common import config
from unload.nes.connection import mysql

class EnrichmentTmcRestrictionsMapper(object):

    def __init__(self, tmcRestrictionsMapper, node_id):
        self.tmcRestrictionsMapper = tmcRestrictionsMapper
        self.node_id = node_id
        self.allGroups = []

    def enrich(self):
        self.normalize_tmc_restrictions_mapper()
        connection = None
        if config.isCash():
            connection = mysql.connect(host='localhost', port=3306, user='netroot', passwd='netroot', db='dictionaries')
        else:
            import urllib.parse as urlparse
            # Чтобы не городить новые настройки считаем подключение к БД продаж. Почти для всех клиентов это решение подойдет
            # Если надо будет, то в будущем можно вынести новую настройку под этот костыль 
            db_connection = config.getString('UNLOAD', 'dbConnection',
                                             'mysql://netroot:netroot@localhost/%(CLUSTER)sdocuments%(DB_SUFFIX)s')
            connectionParams = urlparse.urlparse(db_connection)
            host = connectionParams.hostname if connectionParams.hostname else 'localhost'
            port = connectionParams.port if connectionParams.port else 3306
            user = connectionParams.username if connectionParams.username else 'netroot'
            passwd = connectionParams.password if connectionParams.password else 'netroot'
            connection = mysql.connect(host=host, port=port, user=user, passwd=passwd, db='dictionaries' + self.node_id)
        cursor = connection.cursor()
        cursor.execute("SELECT LOWER(groupcode) as code, LOWER(parentgroupcode) as parentcode FROM tmcgroups")
        self.allGroups = cursor.fetchall()
        rootGroups = [g for g in self.allGroups if g.get('parentcode') is None]
        for g in rootGroups:
            self.fill_childs(g)

    # если в tmcRestrictionsMapper в ключах перечислены коды групп через запятую, то создадим "плоскую" мапу с отдельными кодами групп в ключах
    # и сразу в значениях преобразуем строки в словари
    def normalize_tmc_restrictions_mapper(self):
        tmcRestrictionsMapperTemp = {}
        for k in self.tmcRestrictionsMapper:
            restrictions = {}
            try:
                restrictions = json.loads(self.tmcRestrictionsMapper.get(k))
                restrictions = {str(k).lower(): v for k, v in restrictions.items()}
            except Exception:
                restrictions = {}
            for key in str(k).replace(" ", "").split(','):
                tmcRestrictionsMapperTemp[key] = restrictions
        for k in tmcRestrictionsMapperTemp:
            self.tmcRestrictionsMapper[k] = tmcRestrictionsMapperTemp[k]
        
    def fill_childs(self, parent_group):
        child_groups = [g for g in self.allGroups if g.get('parentcode') == parent_group.get('code')]
        if child_groups:
            for child_group in child_groups:
                if child_group.get('code') not in self.tmcRestrictionsMapper and parent_group.get('code') in self.tmcRestrictionsMapper:
                    self.tmcRestrictionsMapper[child_group.get('code')] = self.tmcRestrictionsMapper.get(parent_group.get('code'))
                self.fill_childs(child_group)
                