# -*- coding: utf-8 -*-

import base64
import httplib2
import json
import re
from httplib2.socks import HTTPError
from upload.node.node import Node, QueueInserterConfig, QueueParameters, SimpleFileConverterParameters
from upload.node.convertergetter import getConverterByName
from common import config, logger, macros
from common.csconvertersconfig import CSConvertersConfig
log = logger.getUploadLogger("main")


def getAllNodeForMiddle(http, base_url, jsonHeader, cashserverId):
    """
    Возвращает специальную ноду All позволяющую произвести загрузку в БД All при загрузке на МКС
    Настройки обмена берутся из МКС
    """
    response, content = http.request(base_url+'/middleServers/%s' % cashserverId, 'GET', headers=jsonHeader)
    if response.status == 200:
        server = json.loads(content)
        server['shopCode'] = 'All'
        server['id'] = 'All'
        return [server]
    return None


def getNodes():
    fileQueueRoot = config.getString('FILEQUEUE', 'fileQueueRoot', '/opt/queues')
    queueChunkSize = config.getInt('FILEQUEUE', 'queueChunkSize', 25000)
    cs_host = config.getString('CS', 'node.host', 'localhost')
    cs_port = config.getString('CS', 'node.port', '8080')
    cs_login = config.getString('CS', 'login', 'admin')
    cs_password = config.getString('CS', 'passwd', 'admin')
    cs_prefix = config.getString('CS', 'node.prefix', '/CSrest/rest')

    auth = base64.b64encode('{0}:{1}'.format(cs_login, cs_password).encode('utf-8')).decode()
    jsonHeader = {'Content-Type': 'application/json', "Accept": "application/json", 'Authorization': 'Basic %s' % auth}

    nodes = []

    categories = []
    log.debug('read nodes from host:%s' % cs_host)
    http = httplib2.Http(timeout=60)
    base_url = 'http://' + cs_host + ':' + cs_port+cs_prefix

    response, content = http.request(base_url + '/system/mode', 'GET', headers=jsonHeader)
    if response.status == 200:
        info = json.loads(content)
        mode = info.get('hierarhyMode')
        cashserverId = info.get('cashserverId')
    else:
        raise HTTPError(f"Request to {base_url}/system/mode returned an error: {response.status} {response.reason}")

    response, content = http.request(base_url + '/categories', 'GET', headers=jsonHeader)
    if response.status == 200:
        categories = json.loads(content)
    middleServers = []
    if "CENTRAL" == mode:
        response, content = http.request(base_url + '/middleServers', 'GET', headers=jsonHeader)
        if response.status == 200:
            middleServers = json.loads(content)

    response, content = http.request(base_url + '/shops', 'GET', headers=jsonHeader)

    if response.status == 200:
        shops = json.loads(content)
        if "CENTRAL" == mode:
            shops = [shop for shop in shops if shop.get('cashserverId') == cashserverId or shop.get('id') == 'All']
        shops += categories
        shops += middleServers
        if "MIDDLE" == mode:
            middleAll = getAllNodeForMiddle(http, base_url, jsonHeader, cashserverId)
            if middleAll:
                shops += middleAll

        for shop in shops:
            shopId = str(shop.get('id'))
            shopState = str(shop.get('state', 'READY'))
            # пропускаем магазины, которые не готовы к работе
            if shopState != 'READY':
                log.info("Object %s with state %s not added to processed" % (shopId, shopState))
                continue
            converters = []
            shopCode = str(shop.get('shopCode', shop.get('middleServerCode', shop.get('displayCode'))))
            clusterId = str(shop.get('clusterId', ''))
            if not clusterId or clusterId == 'None':
                clusterId = ''
            for exchange in shop.get('exchanges', []):
                if not exchange:
                    continue
                csConvertersConfig = CSConvertersConfig(exchange)
                flags = macros.applyMacros(csConvertersConfig.getString('flagPaths', ''), log, shopcode=shopCode, shopid=shopId, clusterid=clusterId)
                flags = [ f.strip() for f in re.split('[,;]', flags) ]
                data = macros.applyMacros(csConvertersConfig.getString('dictPaths', ''), log, shopcode=shopCode, shopid=shopId, clusterid=clusterId)
                data = [ d.strip() for d in re.split('[,;]', data) ]

                backupPath = csConvertersConfig.getString('backupDir', '../backup/dicts/%(clusterid)s/%(shopcode)s')
                backupDir = macros.applyMacros(backupPath, log, shopcode=shopCode, shopid=shopId, clusterid=clusterId)
                errorDest = macros.applyMacros(csConvertersConfig.getString('errorDest', ''), log, shopcode=shopCode, shopid=shopId, clusterid=clusterId)
                exchangeType = exchange.get('type', 'DEFAULT')
                if exchangeType in ['DEFAULT', 'ARTIXINTERCHANGE_SOAP', 'ARTIXINTERCHANGE_REST']:
                    continue
                elif exchangeType == 'SIMPLE-FILE':
                    cashPath = csConvertersConfig.getString('cashPath', '/tmp/upload_files/dataFile.xml')
                    cashPath = macros.applyMacros(cashPath, log, shopcode=shopCode, shopid=shopId, clusterid=clusterId)
                    converters.append(SimpleFileConverterParameters(flags=flags, data=data, backup_dir=backupDir, cashPath=cashPath, cashCodes=shop.get("cashes", [])))
                else:
                    if exchangeType.startswith('SUPERMAG') and exchangeType != 'SUPERMAG_XML':
                        format = csConvertersConfig.getString('format', 'dat')
                        for i in range(len(data)):
                            data[i] = '%s/*.%s' % (data[i], format)
                    converter = getConverterByName(exchangeType, csConvertersConfig, flags=flags, data=data, backupDir=backupDir, errorDest=errorDest)
                    if converter is None:
                        log.warning("Exchange type %s not suported. shopId = %s" % (exchangeType, shopId))
                        continue
                    else:
                        converters.append(converter)

            queueName = 'artix.loaddict.%s' % shopId
            if clusterId != '':
                queueName = queueName + '.' + clusterId
            for converter in converters:
                converter.statusServer = defaultStatusServer

            node = Node(id=shopId,
                        backup_dir=None,
                        converters=converters,
                        pack_size=queueChunkSize,
                        inserter=QueueInserterConfig(queue=QueueParameters(fileQueueRoot, queueChunkSize, queueName)),
                        badDictsDirectory=macros.applyMacros(config.getString('UPLOAD', 'badDictsDirectory', '/opt/exchangers/badDicts').strip(), log, shopcode=shopCode, shopid=shopId, clusterid=clusterId),
                        shopCode=shopCode)
            node.nesServer = defaultNesServer
            nodes.append(node)
    http.close()
    return nodes
